module Guess where

import Control.Exception(catch, IOException)
import System.Random(randomRIO)
import Data.Char (isAlpha, toLower)
import System.Environment(getArgs)

-- Global constant: number of unsuccessfull tries allowed.
numTries :: Int
numTries = 8

-- Utility function to count how often an element occurs in a list
count :: Eq alpha=> alpha-> [alpha]-> Int
count a = length . filter (a ==)

-- Utility function which picks a random element from a (non-empty) list
pickRandom :: [alpha]-> IO alpha
pickRandom [] = error "pickRandom: empty list"
pickRandom xs = do 
  i <- randomRIO (0, length xs - 1)
  return $ xs !! i

render :: String-> String-> String
render rev = map (\c-> if elem c rev then c else '_')

getGuess :: String-> String-> IO String
getGuess hits misses = do
  putStr $ "Your guess? "
  i<- getLine 
  if i == "?" then do 
    putStrLn $ "Your misses so far: "++ filter isAlpha misses
    getGuess hits misses
  else if (length i == 1 && elem (head i) hits) then do
    putStrLn "You already guessed that."
    getGuess hits misses
  else return i

play :: String-> String-> String-> IO ()
play word hits misses
  | length misses == numTries = do 
      putStrLn $ "The word is "++ word++ " -- you lose."
  | otherwise = do
      putStrLn (render hits word)
      putStrLn $ show (length misses) ++ " misses of "++ show numTries++ " tries."
      g <- getGuess hits misses
      case g of 
        [c] | count c word == 0 -> do putStrLn "Miss."; play word hits (c:misses)
        [c] | otherwise -> play word (c:hits) misses
        guess | guess == word -> putStrLn "Excellent, that's correct."
        guess | otherwise -> do putStrLn "Oh no... !"; play word hits (' ':misses)

main :: IO ()
main = 
  catch (do 
    as <- getArgs
    let df= case as of 
                ["de"] -> "dict-de_DE"
                ["en"] -> "dict-en_EN" 
                _ -> "dict-de_DE"
    dict <- readFile df 
    putStrLn $ "Selected dictionary "++ df ++ " ("++ show (length (lines dict))++ " entries)"
    secret <- pickRandom (filter ((5 <). length) (lines dict))
    play (map toLower secret) "" "")
    (\e-> putStrLn $ "Error: "++ show (e:: IOException))

